#include "tests/ListTests.h"
#include "tests/Test.h"
#include "utils/List.h"
#include "utils/StringBuffer.h"

typedef List<int, 20> IntList;
typedef StringBuffer<50> String;

static void testAdd(Test& test) {
    IntList list;
    list.add(5);

    test.checkEqual(5, list[0], "list contains added value");
    test.checkEqual(1, list.getLength(), "list sizes is increased by add");
}

static void testMultipleAdd(Test& test) {
    IntList list;
    list.add(4).add(3).add(2);
    test.checkEqual(4, list[0], "list contains added value");
    test.checkEqual(3, list[1], "list contains added value");
    test.checkEqual(2, list[2], "list contains added value");
    test.checkEqual(3, list.getLength(), "list sizes is increased by add");
}

static void testAddReplace(Test& test) {
    IntList list;
    list.add(5);
    list[0] = 3;
    test.checkEqual(3, list[0], "list value is overwritten");
}

static void testClear(Test& test) {
    IntList list;
    list.add(5).add(4).clear();
    test.checkEqual(0, list.getLength(), "list length is 0 after clear");
}

static void testOverflow(Test& test) {
    IntList list;
    for(int i = 0; i < 1000000; i++) {
        list.add(i);
    }
    for(int i = 0; i < list.getLength(); i++) {
        test.checkEqual(i, list[i], "list still contains values after overflow");
    }
    test.checkEqual(true, true, "list survives overflow");
}

static void testCopy(Test& test) {
    IntList list;
    list.add(1).add(2).add(3);

    IntList copy(list);
    test.checkEqual(list.getLength(), copy.getLength(), "list copy has same length");
    for(int i = 0; i < copy.getLength() && i < list.getLength(); i++) {
        test.checkEqual(list[i], copy[i], "list copy has same values");
    }
}

static void testCopyAssignment(Test& test) {
    IntList list;
    list.add(1).add(2).add(3);

    IntList copy;
    copy = list;
    test.checkEqual(list.getLength(), copy.getLength(), "list copy assignment has same length");
    for(int i = 0; i < copy.getLength() && i < list.getLength(); i++) {
        test.checkEqual(list[i], copy[i], "list copy assignment has same values");
    }
}

static void testMove(Test& test) {
    IntList list;
    list.add(1).add(2).add(3);

    IntList move(std::move(list));
    test.checkEqual(0, list.getLength(), "moved list has length 0");
    test.checkEqual(3, move.getLength(), "moved list passes length");
    test.checkEqual(1, move[0], "moved list passes values");
    test.checkEqual(2, move[1], "moved list passes values");
    test.checkEqual(3, move[2], "moved list passes values");
}

static void testMoveAssignment(Test& test) {
    IntList list;
    list.add(1).add(2).add(3);

    IntList move;
    move = std::move(list);
    test.checkEqual(0, list.getLength(), "assignment moved list has length 0");
    test.checkEqual(3, move.getLength(), "assignment moved list passes length");
    test.checkEqual(1, move[0], "assignment moved list passes values");
    test.checkEqual(2, move[1], "assignment moved list passes values");
    test.checkEqual(3, move[2], "assignment moved list passes values");
}

static void testToString1(Test& test) {
    IntList list;
    list.add(1).add(243).add(-423);
    test.checkEqual(String("[1, 243, -423]"), String(list), "list to string 1");
}

static void testToString2(Test& test) {
    IntList list;
    list.add(1);
    test.checkEqual(String("[1]"), String(list), "list to string 2");
}

static void testToString3(Test& test) {
    IntList list;
    test.checkEqual(String("[]"), String(list), "list to string 3");
}

void ListTests::test() {
    Test test("List");
    testAdd(test);
    testMultipleAdd(test);
    testAddReplace(test);
    testClear(test);
    testOverflow(test);
    testCopy(test);
    testCopyAssignment(test);
    testMove(test);
    testMoveAssignment(test);
    testToString1(test);
    testToString2(test);
    testToString3(test);
    test.finalize();
}